import poser

# Global variables
EZBOOL, EZBUTTON, EZINT, EZCOLOUR, EZCOLOURP, EZFLOAT, EZCHOICE, EZSELECT, EZISELECT, EZFILE, EZLABEL, EZTEXT, EZPRESETS, EZDIR = range(14)
SKINTYPE, EYETYPE, NAILTYPE, MOUTHTYPE, TEETHTYPE, OTHERTYPE = range(6)
METHOD, SHADERTYPE = range(2)

# This class must exist - EZSkin instantiates it for building the shaders
class EZBuilder(object):

	def __init__(self, api):

		self.api = api
		self.parameters = None

		self.has_controls = True
		self.unique_name = '02_EZGhost'

		self.shaders = {
			'ezghost' : (self.EZGhost, OTHERTYPE),
			}

	def _PrepareRoot(self, mat, p):

		# Get original CyclesSurface root node
		n000 = mat.roots['superfly']
		if n000:
			if n000.Type() != poser.kNodeTypeCodePOSERSURFACE:
				n000 = None

		# Make new root
		if p('make_root', 'settings') or n000 is None:
			n000 = mat.MakeRoot(renderer='superfly', roottype=poser.kNodeTypeCodePOSERSURFACE)
			mat.SetPSRoot(n000)

		return n000

	def BuilderName(self):
		return 'EZGhost (SF)'

	def BuildShader(self, material, data, mat):

		# Parse the data
		shader = data[0]
		scattergroup = int(data[1])

		# Don't waste time if we don't have to do anything
		if shader == 'ignore': return 'ignored'

		# Stop if no PoserSurface is found
		if mat.Root() is None: return 'skipped - No Firefly PoserSurface found'

		# Configure imagemap nodes
		p = self.parameters
		if p('gc'): mat.GC_AllMaps()
		if p('filter'): mat.ApplyTextureFilters(p('filtering') + 1)

		# ... and build the shader
		if shader in self.Shaders():

			# Get the method which will build the shader
			method = self.shaders[shader][METHOD]

			# Check to see if we need to use a built-in shader
			if method is None and self.api.Build() >= 68:
				builder = self.api.GetBuilder(self.Renderer())
				if builder:
					return builder.BuildShader(material, data, mat)
				else:
					return 'failed'

			else:
				# Build the shader
				if method is not None:
					method(mat, p)
					mat.RemoveOrphans()
					mat.UpdatePreview()

				return 'processed'

	def ButtonExample(self, event):
		print('Button pressed')

	def Controls(self):

		# Define the controls
		data = (
			('sg01_colour', 'Colour', EZCOLOURP, (None), (1.0, 1.0, 1.0), None, 'The colour of the ghost', 'Main'),
			('sg01_bright', 'Brightness', EZFLOAT, (0.0, 10.0), 1.0, 0.1, 'The brightness of the ghostly colour', 'Main'),
			('sg01_atten', 'Attenuation', EZFLOAT, (0.0, 2.0), 0.5, 0.1, 'Lower values give a finer outline', 'Main'),
			('sg01_shadows', 'Remove shadows', EZBUTTON, (None), self.RemoveShadows, None, 'Stops the entire figure form casting shadows', 'Main'),
			)

		return data

	def Nix(self, mat, p):
		pass

	def RemoveShadows(self, event):
		# Process the figure
		target = self.api.EZSKin().GetActor()
		if target:
			if type(target) == poser.FigureType:
				for act in target.Actors():
					act.SetCastsShadows(0)
			else:
				target.SetCastsShadows(0)

			self.api.SetStatus('Shadow casting stopped for %s' % target.Name())

	def Shaders(self):
		# Used internally by EZSkin
		return list(self.shaders.keys())

	def ShaderType(self, shader):
		# Used internally by EZSkin
		return self.shaders.get(shader, (None, 0))[SHADERTYPE]

	# Our single shader building method
	def EZGhost(self, mat, p):

		# Prepare the root node
		n000 = self._PrepareRoot(mat, p)

		# Prepare for new shader
		mat.ClearPoserSurface(keepTrans=False, keepBump=False, use_root=n000, backup=False)
		mat.SetPSValue('Ambient_Color', p('sg01_colour'))
		mat.SetPSValue('Ambient_Value', p('sg01_bright'))
		mat.SetPSValue('Refraction_Color', (1.0, 1.0, 1.0))
		mat.SetPSValue('Refraction_Value', 1.0)

		# Make nodes
		n001 = mat.AddNode('math_functions', name='Math_Functions', loc=(250, 114), collapsed=False, preview=False, Math_Argument=2.000, Value_1=1.000, Value_2=1.000, )
		n002 = mat.AddNode('edge_blend', name='Edge_Blend', loc=(250, 346), collapsed=False, preview=False, Inner_Color=(1.000, 1.000, 1.000), Outer_Color=(0.000, 0.000, 0.000), Attenuation=p('sg01_atten'), )
		n003 = mat.AddNode('refract', name='Refract', loc=(250, 213), collapsed=False, preview=False, Background=(0.000, 0.000, 0.000), Index_of_Refraction=1.000, Quality=0.200, Diffusion=0.000, RayBias=0.850, )


		# Connect nodes
		mat.ConnectNodes(n001, n000, 'Ambient_Value')
		mat.ConnectNodes(n003, n000, 'Refraction_Color')
		mat.ConnectNodes(n002, n000, 'Refraction_Value')
		mat.ConnectNodes(n002, n001, 'Value_2')

	def Renderer(self):
		# Return the render engine this plugin is designed to work with
		# Must be 'Firefly' or 'Superfly'
		return 'Superfly'
